#include<iostream>
#include<cstdlib>
#include<cstring>
#include<vector>

using namespace std;

const int MAXN = 1e6 + 5;
const long long inf = 1e17 + 5;

struct node
{
    long long maxValue1, maxValue2;

    int l, r;
    node *L, *R;

    node(){}
    node(int l, int r)
    {
        this->maxValue1 = -inf;
        this->maxValue2 = -inf;

        this->l = l;
        this->r = r;
        this->L = nullptr;
        this->R = nullptr;
    }

    void build()
    {
        if(this->l==this->r)
        {
            return;
        }

        this->L = new node(this->l, (this->l + this->r)/2);
        this->R = new node((this->l + this->r)/2+1, this->r);

        this->L->build();
        this->R->build();
    }

    long long query1(int ql, int qr)
    {
        if(ql>qr) return -inf;

        if(this->l>=ql && this->r<=qr) return this->maxValue1;
        if(this->r<ql || this->l>qr) return -inf;

        return max(this->L->query1(ql, qr), this->R->query1(ql, qr));
    }

    long long query2(int ql, int qr)
    {
        if(ql>qr) return -inf;

        if(this->l>=ql && this->r<=qr) return this->maxValue2;
        if(this->r<ql || this->l>qr) return -inf;

        return max(this->L->query2(ql, qr), this->R->query2(ql, qr));
    }

    void update(int q, long long dist, long long depth)
    {
        if(this->l==this->r && this->l==q)
        {
            this->maxValue1 = depth - dist;
            this->maxValue2 = depth + dist;

            return;
        }
        if(q<this->l || q>this->r) return;

        this->L->update(q, dist, depth);
        this->R->update(q, dist, depth);

        this->maxValue1 = max(this->L->maxValue1, this->R->maxValue1);
        this->maxValue2 = max(this->L->maxValue2, this->R->maxValue2);
    }
};

long long maxValue1[MAXN], maxValue2[MAXN];

bool isCycle[MAXN];
int parent[MAXN];

bool used[MAXN];
vector < pair <int, int> > graph[MAXN];

long long answer = 0;
long long maxDepth[MAXN], maxSum[MAXN];

node *T;

void DFSInit(int x, int last)
{
    used[x] = true;
    //cout << x << '\n';

    for(int i = 0;i<graph[x].size();i++)
    {
        if(graph[x][i].first==x) continue;

        if(used[ graph[x][i].first ]==false)
        {
            parent[ graph[x][i].first ] = x;
            DFSInit(graph[x][i].first, x);
        }
        else if(graph[x][i].first!=last && isCycle[ graph[x][i].first ]==false)
        {
            //cout << " -- " << graph[x][i].first << '\n';

            int u = x;
            while(true)
            {
                //cout << u << '\n';
                //system("PAUSE");

                isCycle[u] = true;
                if(u==graph[x][i].first) break;

                u = parent[u];
            }
        }
    }
}

void DFSTree(int x, int last)
{
    maxSum[x] = 0;
    maxDepth[x] = 0;
    for(int i = 0;i<graph[x].size();i++)
    {
        if(graph[x][i].first==x) continue;

        if(isCycle[ graph[x][i].first ]==false && graph[x][i].first!=last)
        {
            DFSTree(graph[x][i].first, x);
        }
    }

    int index1, index2;
    long long max1 = 0, max2 = 0;

    for(int i = 0;i<graph[x].size();i++)
    {
        if(graph[x][i].first==x) continue;

        if(isCycle[ graph[x][i].first ]==false && graph[x][i].first!=last)
        {
            maxSum[x] = max(maxSum[x], maxSum[ graph[x][i].first ]);
            maxDepth[x] = max(maxDepth[x], maxDepth[ graph[x][i].first ]+graph[x][i].second);

            if(maxDepth[ graph[x][i].first ]+graph[x][i].second > max1)
            {
                index1 = i;
                max1 = maxDepth[ graph[x][i].first ]+graph[x][i].second;
            }
        }
    }
    for(int i = 0;i<graph[x].size();i++)
    {
        if(graph[x][i].first==x) continue;

        if(isCycle[ graph[x][i].first ]==false && graph[x][i].first!=last && index1!=i)
        {
            if(maxDepth[ graph[x][i].first ]+graph[x][i].second > max2)
            {
                max2 = maxDepth[ graph[x][i].first ]+graph[x][i].second;
            }
        }
    }

    maxSum[x] = max(maxSum[x], max1 + max2);
    maxSum[x] = max(maxSum[x], maxDepth[x]);

    answer = max(answer, max1+max2);
}

int p[MAXN], w[MAXN];
vector < pair <int, long long> > v;

long long cycleSum = 0;

int findIndex(int currIndex, long long x)
{
    int l, r, mid;

    l = 0;
    r = currIndex - 1;
    while(l+1<r)
    {
        //cout << l << " --- " << r << '\n';

        mid = (l+r)/2;
        long long dist = x - ((mid==0)?0:v[mid-1].second);

        if(dist <= cycleSum - dist) r = mid;
        else l = mid + 1;
    }
    //cout << "oke" << '\n';

    long long dist = x - ((l==0)?0:v[l-1].second);
    if(dist <= cycleSum - dist) return l;

    dist = x - ((r==0)?0:v[r-1].second);
    if(dist <= cycleSum - dist) return r;

    return -1;
}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    int n;
    cin >> n;

    for(int i = 1;i<=n;i++)
    {
        cin >> p[i];
    }
    for(int i = 1;i<=n;i++)
    {
        cin >> w[i];
    }
    for(int i = 1;i<=n;i++)
    {
        graph[i].push_back({p[i], w[i]});
        graph[ p[i] ].push_back({i, w[i]});
    }

    DFSInit(1, -1);

    int u = -1;
    for(int i = 1;i<=n;i++)
    {
        if(isCycle[i]==true)
        {
            u = i;
            break;
        }
    }

    if(u==-1)
    {
        DFSTree(1, -1);
        cout << answer << '\n';

        return 0;
    }

    int firstNode = u;
    memset(used, false, sizeof(used));

    while(true)
    {
        used[u] = true;

        bool succes = false;
        for(int i = 0;i<graph[u].size();i++)
        {
            if(used[ graph[u][i].first ]==false && isCycle[ graph[u][i].first ]==true)
            {
                succes = true;

                cycleSum += graph[u][i].second;
                v.push_back({u, cycleSum});
                u = graph[u][i].first;

                break;
            }
        }

        if(succes==false)
        {
            for(int i = 0;i<graph[u].size();i++)
            {
                if(graph[u][i].first==firstNode)
                {
                    cycleSum += graph[u][i].second;
                    v.push_back({u, cycleSum});


                    break;
                }
            }

            break;
        }
    }

    //cout << cycleSum << '\n';
    for(int i = 0;i<v.size();i++)
    {
        //cout << v[i].first << " " << v[i].second << '\n';
    }

    for(int i = 0;i<v.size();i++)
    {
        DFSTree(v[i].first, -1);
    }

    /*
    for(int i = 0;i<v.size();i++)
    {
        for(int j = i+1;j<v.size();j++)
        {
            long long dist = v[j-1].second;
            if(i>0) dist -= v[i-1].second;

            answer = max(answer, maxDepth[ v[i].first ] + maxDepth[ v[j].first ] + min(dist, cycleSum - dist ));
            //cout << v[i].first << " -> " << v[j].first << " = " << maxDepth[ v[i].first ] << " + " << maxDepth[ v[j].first ]
            // << " + " << min(dist, cycleSum - dist ) << '\n';
        }
    }
    */

    T = new node(0, v.size()-1);
    T->build();

    maxValue1[0] = maxDepth[ v[0].first ] - 0;
    maxValue2[0] = maxDepth[ v[0].first ] + 0;

    T->update(0, 0, maxDepth[ v[0].first ]);
    for(int i = 1;i<v.size();i++)
    {
        long long add;
        int index = findIndex(i, v[i-1].second);
        //cout << "OKE " << index << '\n';

        if(index==-1)
        {
            /*
            add = -inf;
            for(int j = 0;j<i;j++)
            {
                add = max(add, maxValue2[j]);
            }
            */

            add = T->query2(0, i-1);
            answer = max(answer, v[i-1].second + maxDepth[ v[i].first ] + add);
        }
        else
        {
            //cout << "START" << '\n';

            /*
            long long add = -inf;
            for(int j = index;j<i;j++)
            {
                add = max(add, maxValue1[j]);
            }
            */

            add = T->query1(index, i-1);
            answer = max(answer, v[i-1].second + maxDepth[ v[i].first ] + add);

            /*
            add = -inf;
            for(int j = 0;j<index;j++)
            {
                add = max(add, maxValue2[j]);
            }
            */

            add = T->query2(0, index-1);
            answer = max(answer, cycleSum - v[i-1].second + maxDepth[ v[i].first ] + add);
            //cout << "END" << '\n';
        }

        maxValue1[i] = maxDepth[ v[i].first ] - v[i-1].second;
        maxValue2[i] = maxDepth[ v[i].first ] + v[i-1].second;

        T->update(i, v[i-1].second, maxDepth[ v[i].first ]);
    }

    cout << answer << '\n';
}
/*
8
2 4 4 1 1 5 5 7
1 2 3 4 5 6 7 8
*/
