#include <bits/stdc++.h>

const int64_t MAX_N = 1e6;

struct SegmentTree {
	int64_t data[4 * MAX_N + 5];

	void update(int64_t node, int64_t low, int64_t high, int64_t ind, int64_t val) {
		if(low > ind || high < ind) {
			return;
		}
		else if(low == ind && high == ind) {
			data[node] = val;
			return;
		}

		int64_t mid = (low + high) / 2;
		update(2 * node, low, mid, ind, val);
		update(2 * node + 1, mid + 1, high, ind, val);

		data[node] = std::max(data[2 * node], data[2 * node + 1]);
	}

	int64_t query(int64_t node, int64_t low, int64_t high, int64_t qLow, int64_t qHigh) {
		if(low > qHigh || high < qLow) {
			return 0;
		}
		else if(low >= qLow && high <= qHigh) {
			return data[node];
		}

		int64_t mid = (low + high) / 2;
		return std::max(query(2 * node, low, mid, qLow, qHigh), query(2 * node + 1, mid + 1, high, qLow, qHigh));
	}
};

SegmentTree segTree;
	
struct Graph {
	bool vis[MAX_N + 5], cycle[MAX_N + 5];
	int64_t n, par[MAX_N + 5], startOfCycle, endOfCycle;
	std::vector< std::pair< int64_t, int64_t > > v[MAX_N + 5];
	std::stack< int64_t > toClear;
	
	void init(int64_t _n) {
		n = _n;
		startOfCycle = -1;
	}

	void add_edge(int64_t p, int64_t q, int64_t w) {
		v[p].push_back({ q, w });
		v[q].push_back({ p, w });
	}

	std::pair< int64_t, int64_t > find_furthest_tree(int64_t x) {
		vis[x] = true;
		toClear.push(x);

		std::pair< int64_t, int64_t > ans = { x, 0 };
	
		for(auto &p : v[x]) {
			if(!vis[p.first] && !cycle[p.first]) {
				auto aux = find_furthest_tree(p.first);
				aux.second += p.second;
				
				if(aux.second > ans.second) {
					ans = aux;
				}
			}
		}

		return ans;
	}

	int64_t solve_tree(int64_t x = 1) {
		while(!toClear.empty()) {
			vis[toClear.top()] = false;
			toClear.pop();
		}
		
		auto furthest1 = find_furthest_tree(x);
		
		while(!toClear.empty()) {
			vis[toClear.top()] = false;
			toClear.pop();
		}
		
		auto furthest2 = find_furthest_tree(furthest1.first);
		return furthest2.second;
	}
	
	void dfs_init(int64_t x) {
		vis[x] = true;
		
		for(auto &p : v[x]) {
			if(!vis[p.first]) {
				par[p.first] = x;
				dfs_init(p.first);
			}
			else if(p.first != par[x] && startOfCycle == -1) {
				startOfCycle = p.first;
				endOfCycle = x;
			}
		}
	}

	int64_t solve_normal() {
		dfs_init(1);
		memset(vis, 0, sizeof(vis));

		int64_t ans = 0;
	
		std::vector< int64_t > cycleVertices;
		std::vector< int64_t > furthest;
		
		for(int64_t x = endOfCycle; x != startOfCycle; x = par[x]) {
			cycleVertices.push_back(x);
			cycle[x] = true;
		}
		cycleVertices.push_back(startOfCycle);
		cycle[startOfCycle] = true;
	
		std::vector< int64_t > d;
		for(int64_t i = 0; i < cycleVertices.size(); i++) {
			int64_t target;
			if(i != cycleVertices.size() - 1) {
				target = cycleVertices[i + 1];
			}
			else {
				target = cycleVertices[0];
			}

			for(auto &x : v[cycleVertices[i]]) {
				if(x.first == target) {
					d.push_back(x.second);
					break;
				}
			}

			ans = std::max(ans, solve_tree(cycleVertices[i]));
			
			while(!toClear.empty()) {
				vis[toClear.top()] = false;
				toClear.pop();
			}
			furthest.push_back(find_furthest_tree(cycleVertices[i]).second);
		}

		std::vector< int64_t > prefSums(d.size());
		for(int64_t i = 0; i < d.size(); i++) {
			if(i == 0) {
				prefSums[i] = 0;
			}
			else {
				prefSums[i] = prefSums[i - 1] + d[i - 1];
			}
		}

		int64_t sum = prefSums[d.size() - 1] + d[d.size() - 1];
		ans = std::max(ans, sum);

		int64_t currD = 0;
		std::vector< int64_t > best(d.size());
		for(int64_t i = 0; i < d.size(); i++) {
			segTree.update(1, 1, d.size(), i + 1, currD + furthest[i]);
			currD += d[i];
		}

		for(int64_t i = 0; i < d.size(); i++) {
			int64_t low = 0, high = d.size() - 1;
			while(low <= high) {
				int64_t mid = (low + high) / 2;

				int64_t other = i + mid;
				
				int64_t curr;
				if(other >= d.size()) {
					curr = sum - prefSums[i] + prefSums[other - d.size()];
				}
				else {
					curr = prefSums[other] - prefSums[i];
				}

				if(sum - curr < curr) {
					high = mid - 1;
				}
				else {
					low = mid + 1;
				}
			}
			
			int64_t other = i + low - 1;

			if(other == i) {
				continue;
			}

			std::cout << i << " " << other << '\n';

			if(other < d.size()) {
				ans = std::max(ans, segTree.query(1, 1, d.size(), i + 1, other + 1) - prefSums[i] + furthest[i]);
			}
			else {
				ans = std::max(ans, segTree.query(1, 1, d.size(), i + 1, d.size()) - prefSums[i] + furthest[i]);
				ans = std::max(ans, segTree.query(1, 1, d.size(), 1, other - d.size() + 1) + (sum - prefSums[i]) + furthest[i]);
			}
		}

		return ans;
	}
};

int64_t p[MAX_N + 5], w[MAX_N + 5];
Graph g;

int main() {
	std::ios_base::sync_with_stdio(false);
	std::cin.tie(nullptr);

	int64_t n;
	std::cin >> n;

	for(int64_t i = 1; i <= n; i++) {
		std::cin >> p[i];
	}
	for(int64_t i = 1; i <= n; i++) {
		std::cin >> w[i];
	}

	g.init(n);

	bool isTree = false;
	for(int64_t i = 1; i <= n; i++) {
		if(i == p[i]) {
			isTree = true;
		}
		else {
			g.add_edge(i, p[i], w[i]);
		}
	}

	if(isTree) {
		std::cout << g.solve_tree() << '\n';
	}
	else {
		std::cout << g.solve_normal() << '\n';
	}
}
