#pragma GCC optimize "Ofast"
#include <iostream>
#include <queue>
#include <random>
#include <ctime>

#define D(x) std::cout << __LINE__ << ": " << #x << " = " << x << std::endl;
typedef long long ll;

ll n = 0;

const ll MAX_N = 1e6 + 10;

struct edge {
	ll from, to, w;
} edges[MAX_N];

std::vector<std::pair<ll, ll>> graph[MAX_N];

ll dist[MAX_N];
ll e[MAX_N];

void dij(ll start) {
	std::fill(dist, dist+n+10, (1ll<<60));

	std::priority_queue<std::pair<ll, ll>> next;
	next.push({0, start});
	dist[start] = 0;
	while(next.size()) {
		auto curr = next.top();
		next.pop();
		//std::cout << "here " << curr.second << '\n';
		//curr.first *= -1;
		for(auto nei : graph[curr.second]) {
			if(dist[curr.second] + nei.second < dist[nei.first]) {
				dist[nei.first] = dist[curr.second] + nei.second;
				next.push({-dist[nei.first], nei.first});
			}
		}
	}
}

ll sol(int beg) {
	dij(beg);
	ll max = 1;
	for(ll i = 1;i <= n;i ++) {
		if(dist[i] > dist[max]) max = i;
		//std::cout << i << ' ' << dist[i] << '\n';
	}

	dij(max);
	for(ll i = 1;i <= n;i ++) {
		if(dist[i] > dist[max]) max = i;
		//std::cout << i << ' ' << dist[i] << '\n';
	}

	return dist[max];
}

int parent[MAX_N];
int find(int a) {
	if(parent[a] == a) return a;
	return parent[a] = find(parent[a]);
}
void unite(int a, int b) {
	parent[find(a)] = find(b);
}

int main() {
	std::ios_base::sync_with_stdio(false);
	std::cin.tie(nullptr);

	bool tree = false;
	int wa, wb;

	std::cin >> n;
	for(int i = 0;i < n;i ++) {
		parent[i] = i;
	}

	for(ll i = 0;i < n;i ++) {
		edges[i].from = i + 1;
		std::cin >> edges[i].to;
	}
	for(ll i = 0;i < n;i ++) {
		std::cin >> edges[i].w;
	}
	for(ll i = 0;i < n;i ++) {
		const int to = edges[i].to;
		const int fr = edges[i].from;
		if(to == fr) tree = true;
		graph[fr].push_back({to, edges[i].w});
		graph[to].push_back({fr, edges[i].w});
		if(find(to) == find(fr)) {
			//std::cout << "weird " << to << ' ' << fr << std::endl;
			wa = to;
			wb = fr;
		} else {
			unite(to, fr);
		}
	}

	std::random_device rd;
	std::default_random_engine eng(rd() + 108);
	std::uniform_int_distribution<int> dist(1, n);

	ll ans = 0;
	if(tree) {
		ans = sol(1);
	} else {
		ans = std::max(ans, sol(1));
		ans = std::max(ans, sol(wa));
		ans = std::max(ans, sol(wb));
	}

	std::cout << ans << std::endl;

	return 0;
}
